module.exports = {
    'parserOptions': {
        'ecmaVersion': 2017,
        'ecmaFeature': {
            'impliedStrict': true
        }
    },
    'env': {
        'node': true,
        'browser': true,
        'es6': true
    },
    'extends': 'eslint:recommended',
    'rules': {
        //Possible Errors
        'no-console': 'off',
        'no-extra-parens': ['warn', 'functions'],
        'no-template-curly-in-string': 'error',

        //Best Practices
        'accessor-pairs': 'error',
        'array-callback-return': 'error',
        'consistent-return': 'error',
        'curly': ['warn', 'multi-or-nest', 'consistent'],
        'default-case': 'warn',
        'dot-notation': ['error', {allowKeywords: true}],
        'eqeqeq': 'error',
        'no-alert': 'error',
        'no-caller': 'error',
        'no-else-return': 'warn',
        'no-empty-function': 'warn',
        'no-eq-null': 'error',
        'no-eval': 'error',
        'no-extend-native': 'error',
        'no-extra-bind': 'error',
        'no-extra-label': 'warn',
        'no-fallthrough': 'warn',
        'no-implicit-coercion': 'error',
        'no-implicit-globals': 'error',
        'no-implied-eval': 'error',
        'no-iterator': 'error',
        'no-labels': ['error', {allowLoop: true, allowSwitch: true}],
        'no-lone-blocks': 'error',
        'no-loop-func': 'error',
        // 'no-magic-numbers': ['warn', {'ignoreArrayIndexes': true}],
        'no-multi-spaces': 'error',
        'no-multi-str': 'error',
        'no-new': 'error',
        'no-new-wrappers': 'error',
        'no-octal': 'error',
        'no-octal-escape': 'error',
        'no-proto': 'error',
        'no-redeclare': 'error',
        'no-script-url': 'error',
        'no-self-compare': 'error',
        'no-sequences': 'error',
        'no-throw-literal': 'error',
        'no-unused-expressions': 'error',
        'no-useless-call': 'warn',
        'no-useless-concat': 'warn',
        'no-useless-escape': 'warn',
        'no-useless-return': 'warn',
        'no-void': 'error',
        'no-with': 'error',
        'prefer-promise-reject-errors': 'error',
        'require-await': 'error',
        'yoda': ['error', 'never'],

        //Strict Mode
        'strict': ['error', 'safe'],

        //Variables
        'no-catch-shadow': 'error',
        'no-label-var': 'error',
        'no-shadow': ['error', {'hoist': 'never'}],
        'no-shadow-restricted-names': 'error',
        'no-undef': 'error',
        'no-undef-init': 'error',
        'no-unused-vars': ['error', {args: 'none'}],
        'no-use-before-define': ['error', {'functions': false, 'classes': false}],

        //Node.js and CommonJS
        'no-mixed-requires': 'error',
        'no-new-require': 'error',
        'no-path-concat': 'error',
        'no-process-exit': 'error',
        'no-restricted-modules': 'error',
        'no-sync': "error",

        //Stylistic Issues
        'array-bracket-newline': ['warn', {'multiline': true}],
        'array-bracket-spacing': ['warn', 'never', {'arraysInArrays': true}],
        //'array-element-newline': ['warn', {'multiline': true}],
        'block-spacing': 'warn',
        'brace-style': 'warn',
        'camelcase': 'warn',
        'comma-dangle': ['warn', 'never'],
        'comma-style': 'warn',
        'computed-property-spacing': ['error', 'never'],
        // 'consistent-this': ['warn', 'self'],
        'eol-last': ['warn', 'always'],
        'func-call-spacing': ['warn', 'never'],
        'func-name-matching': 'warn',
        // 'func-names': ['warn', 'as-needed'],
        'indent': ['warn', 4, {SwitchCase: 1}],
        'key-spacing': ['warn', {beforeColon: false, afterColon: true}],
        'linebreak-style': ['warn', 'unix'],
        'lines-around-comment': ['warn', {'beforeBlockComment': true}],
        'max-statements-per-line': 'warn',
        'new-cap': ['warn', {'newIsCap': true, 'properties': true}],
        'new-parens': 'warn',
        'newline-per-chained-call': ['warn', {'ignoreChainWithDepth': 3}],
        'no-array-constructor': 'warn',
        'no-lonely-if': 'warn',
        'no-multiple-empty-lines': ['warn', {'max': 2, 'maxBOF': 0, 'maxEOF': 1}],
        'no-new-object': 'warn',
        'no-trailing-spaces': ['warn', {'skipBlankLines': true}],
        'no-unneeded-ternary': 'warn',
        'no-whitespace-before-property': 'error',
        'nonblock-statement-body-position': ['warn', 'below'],
        'object-curly-newline': ['warn', {'multiline': true, 'consistent': true}],
        // 'object-curly-spacing': ['warn', 'never', { 'arraysInObjects': false, 'objectsInObjects': true }],
        'quotes': ['warn', 'single', {"allowTemplateLiterals": true}],
        'semi': ['warn', 'always'],
        'semi-style': ['warn', 'last'],
        'semi-spacing': 'warn',
        'space-before-blocks': 'warn',
        'space-before-function-paren': ['warn', {'anonymous': 'always', 'named': 'never', 'asyncArrow': 'always'}],
        'space-in-parens': ['warn', 'never'],
        'space-infix-ops': ['warn', {'int32Hint': false}],
        'space-unary-ops': 'warn',
        'switch-colon-spacing': ['warn', {'after': true, 'before': false}],

        //ECMAScript 6
        'arrow-body-style': ['warn', 'as-needed'],
        'arrow-parens': ['warn', 'as-needed'],
        'arrow-spacing': 'warn',
        'constructor-super': 'error',
        'generator-star-spacing': ['warn', {'before': true, 'after': false}],
        'no-duplicate-imports': 'error',
        'no-useless-computed-key': 'error',
        'no-var': 'error',
        'object-shorthand': ['warn', 'always', {'avoidQuotes': true}],
        'prefer-arrow-callback': 'warn',
        'prefer-const': ['error', {'destructuring': 'all'}],
        'prefer-numeric-literals': 'error',
        'prefer-rest-params': 'warn',
        'prefer-spread': 'warn',
        'prefer-template': 'warn',
        'rest-spread-spacing': ['warn', 'never'],
        'symbol-description': 'error',
        'template-curly-spacing': 'error',
        'yield-star-spacing': ['error', 'after'],
    }
};